/* jshint latedef:false */
/* jshint forin:false */

// 
// Copyright (c) Microsoft.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

var xml = require('xml2js');
var util = require('util');
var js2xml = require('../../util/js2xml');
var Service = require('../core/service');
var WebResource = require('../../http/webresource');

var SubscriptionClient = (function() {
  /**
   * Initializes a new instance of the SubscriptionClient class.
   *
   * @constructor
   *
   * @param {CloudCredentials} credentials - Credentials used to authenticate
   * requests.
   *
   * @param {String} baseUri - The URI used as the base for all Service
   * Management requests.
   */
  function SubscriptionClient(credentials, baseUri) {
    if (credentials === null || credentials === undefined) {
      throw new Error('credentials cannot be null.');
    }
    
    SubscriptionClient['super_'].call(this, credentials);
    
    this.credentials = credentials;
    this.baseUri = baseUri;
    if (this.baseUri === null || this.baseUri === undefined) {
      this.baseUri = 'https://management.core.windows.net';
    }
    this.subscriptions = new SubscriptionOperations(this);
  }
  
  util.inherits(SubscriptionClient, Service);
  
  
  return SubscriptionClient;
})();
exports.SubscriptionClient = SubscriptionClient;

var SubscriptionOperations = (function() {
  /**
   * Initializes a new instance of the SubscriptionOperations class.
   *
   * @constructor
   *
   * @param {SubscriptionClient} client - Reference to the service client.
   */
  function SubscriptionOperations(client) {
    this.client = client;
  }
  
  /**
   *
   * @param {Function} callback
   *
   * @returns {SubscriptionListOperationResponse} - A standard service response
   * including an HTTP status code and request ID.
   */
  SubscriptionOperations.prototype.list = function(callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var url2 = this.client.baseUri + '/subscriptions';
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/xml;charset=utf-8';
    httpRequest.headers['x-ms-version'] = '2013-08-01';
    
    // Set Credentials
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        return callback(body);
      }
      
      // Create Result
      var result = { subscriptions: [] };
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      // Deserialize Response
      var responseContent = body;
      var options = {};
      options.trim = true;
      options.strict = false;
      return xml.parseString(responseContent, options, function (err2, responseDoc) {
        if (err2) {
          return callback(err2);
        }
        
        var subscriptionsElement = js2xml.getElement(responseDoc, responseDoc, 'SUBSCRIPTIONS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
        if (subscriptionsElement) {
          if (subscriptionsElement) {
            for (var loweredIndex1 = 0; loweredIndex1 < js2xml.getElements(responseDoc, subscriptionsElement, 'SUBSCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE').length; loweredIndex1 = loweredIndex1 + 1) {
              var subscriptionsElement2 = js2xml.getElements(responseDoc, subscriptionsElement, 'SUBSCRIPTION', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE')[loweredIndex1];
              var subscriptionInstance = {};
              result.subscriptions.push(subscriptionInstance);
              
              var subscriptionIDElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'SUBSCRIPTIONID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (subscriptionIDElement) {
                var subscriptionIDInstance = subscriptionIDElement;
                subscriptionInstance.subscriptionId = subscriptionIDInstance;
              }
              
              var subscriptionNameElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'SUBSCRIPTIONNAME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (subscriptionNameElement) {
                var subscriptionNameInstance = subscriptionNameElement;
                subscriptionInstance.subscriptionName = subscriptionNameInstance;
              }
              
              var subscriptionStatusElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'SUBSCRIPTIONSTATUS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (subscriptionStatusElement) {
                var subscriptionStatusInstance = subscriptionStatusElement;
                subscriptionInstance.subscriptionStatus = subscriptionStatusInstance;
              }
              
              var accountAdminLiveEmailIdElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'ACCOUNTADMINLIVEEMAILID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (accountAdminLiveEmailIdElement) {
                var accountAdminLiveEmailIdInstance = accountAdminLiveEmailIdElement;
                subscriptionInstance.accountAdminLiveEmailId = accountAdminLiveEmailIdInstance;
              }
              
              var serviceAdminLiveEmailIdElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'SERVICEADMINLIVEEMAILID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (serviceAdminLiveEmailIdElement) {
                var serviceAdminLiveEmailIdInstance = serviceAdminLiveEmailIdElement;
                subscriptionInstance.serviceAdminLiveEmailId = serviceAdminLiveEmailIdInstance;
              }
              
              var maxCoreCountElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'MAXCORECOUNT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (maxCoreCountElement) {
                var maxCoreCountInstance = parseInt(maxCoreCountElement, 10);
                subscriptionInstance.maximumCoreCount = maxCoreCountInstance;
              }
              
              var maxStorageAccountsElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'MAXSTORAGEACCOUNTS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (maxStorageAccountsElement) {
                var maxStorageAccountsInstance = parseInt(maxStorageAccountsElement, 10);
                subscriptionInstance.maximumStorageAccounts = maxStorageAccountsInstance;
              }
              
              var maxHostedServicesElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'MAXHOSTEDSERVICES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (maxHostedServicesElement) {
                var maxHostedServicesInstance = parseInt(maxHostedServicesElement, 10);
                subscriptionInstance.maximumHostedServices = maxHostedServicesInstance;
              }
              
              var currentCoreCountElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'CURRENTCORECOUNT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (currentCoreCountElement) {
                var currentCoreCountInstance = parseInt(currentCoreCountElement, 10);
                subscriptionInstance.currentCoreCount = currentCoreCountInstance;
              }
              
              var currentStorageAccountsElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'CURRENTSTORAGEACCOUNTS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (currentStorageAccountsElement) {
                var currentStorageAccountsInstance = parseInt(currentStorageAccountsElement, 10);
                subscriptionInstance.currentStorageAccounts = currentStorageAccountsInstance;
              }
              
              var currentHostedServicesElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'CURRENTHOSTEDSERVICES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (currentHostedServicesElement) {
                var currentHostedServicesInstance = parseInt(currentHostedServicesElement, 10);
                subscriptionInstance.currentHostedServices = currentHostedServicesInstance;
              }
              
              var maxVirtualNetworkSitesElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'MAXVIRTUALNETWORKSITES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (maxVirtualNetworkSitesElement) {
                var maxVirtualNetworkSitesInstance = parseInt(maxVirtualNetworkSitesElement, 10);
                subscriptionInstance.maximumVirtualNetworkSites = maxVirtualNetworkSitesInstance;
              }
              
              var maxLocalNetworkSitesElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'MAXLOCALNETWORKSITES', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (maxLocalNetworkSitesElement) {
                var maxLocalNetworkSitesInstance = parseInt(maxLocalNetworkSitesElement, 10);
                subscriptionInstance.maximumLocalNetworkSites = maxLocalNetworkSitesInstance;
              }
              
              var maxDnsServersElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'MAXDNSSERVERS', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (maxDnsServersElement) {
                var maxDnsServersInstance = parseInt(maxDnsServersElement, 10);
                subscriptionInstance.maximumDnsServers = maxDnsServersInstance;
              }
              
              var maxExtraVIPCountElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'MAXEXTRAVIPCOUNT', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (maxExtraVIPCountElement) {
                var maxExtraVIPCountInstance = parseInt(maxExtraVIPCountElement, 10);
                subscriptionInstance.maximumExtraVirtualIPCount = maxExtraVIPCountInstance;
              }
              
              var aADTenantIDElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'AADTENANTID', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (aADTenantIDElement) {
                var aADTenantIDInstance = aADTenantIDElement;
                subscriptionInstance.activeDirectoryTenantId = aADTenantIDInstance;
              }
              
              var createdTimeElement = js2xml.getElement(responseDoc, subscriptionsElement2, 'CREATEDTIME', 'HTTP://SCHEMAS.MICROSOFT.COM/WINDOWSAZURE');
              if (createdTimeElement) {
                var createdTimeInstance = new Date(createdTimeElement);
                subscriptionInstance.created = createdTimeInstance;
              }
            }
          }
        }
        
        return callback(null, result);
      });
    });
  };
  
  return SubscriptionOperations;
})();
